// -- DLL source file (C/C++)-- //

// THK 11/3/98 - This dll is called from uninstallshield to clean up registry/autoexec path settings.
// Currently used for Part Adviser installations only.
// Mold3d and Dynamic Series now set environment variables dynamically and do not write to the registry/autoexec
// Part Adviser 2 installation now contains Part Adviser 2 Alpha as well, so PTA2DA is redundant.
// Dynamic Series, Mold3d and PTA2DA code is still present, although it is not required.

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <ctype.h>
#include <atlbase.h>

#define BUFFER_SIZE 8096
#define MF_PTA2		 1	//Part Adviser 2 Intel and Alpha
#define MF_PTA2_PROE 2  //ProE PLASTIC Advisor 2 Intel and Alpha
#define MF_PTA2_1	 3	//Part Adviser 2.1 Intel and Alpha 

static char dir[BUFFER_SIZE];
static LONG error;
static HWND parent;
static const char *msgs[] = {"Removing software from system-wide PATH.","Removing software from per-user PATH." };
static const char *msg95 [] = {"Removing software from autoexec.bat"};
static int product = 0;

static void remove_dir_from_path(HKEY root, char *subkey, int is_local);
static void do_error(DWORD error, char *msg);
//THK 27/2/98 - prototype for the delete key function
static DWORD RegDeleteKeyNT(HKEY hStartKey , LPTSTR pKeyName ) ;


char g_DebugMessage [ 1024 ] = "" ;
char g_DebugTitle   [ 1024 ] = "" ;
#ifdef _DEBUG
#define DEBUGMSG(TITLE,MSG) MessageBox ( NULL, MSG, TITLE, MB_OK ) ;
#else
#define DEBUGMSG(TITLE,MSG) /* Do nothing */
#endif


/*
__declspec( dllexport )
    int UninstInitialize(HWND hwndDlg, HWND hInstance, long int lReserved);
__declspec( dllexport )
    int UninstUnInitialize(HWND hwndDlg, HWND hInstance, long int lReserved);
*/


HKEY open_key(HKEY root, char *subkey)
{
    DWORD       disposition;
    HKEY        key;
    
    error = RegCreateKeyEx(root, subkey, 0, "", REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS, NULL, &key, &disposition);
    
    if((error != ERROR_SUCCESS) || (disposition != REG_OPENED_EXISTING_KEY))
        return (HKEY)0;
    
    return key;
}

void remove_dir_from_path(HKEY root, char *subkey, int is_local)
{ 
    HKEY    key;
    char    oldpath[BUFFER_SIZE], newpath[BUFFER_SIZE];
    DWORD   size = BUFFER_SIZE;
    char    *ii;
    int     jj;
	
	// THK 15/12/97 - Check if DLL is called from UninstallShield
	DEBUGMSG( "DLL called.","DLL Initialized" ) ;
	sprintf ( g_DebugTitle,   "DLL Initialized" ) ;
	sprintf ( g_DebugMessage, "Sub Key is %s",subkey);
	DEBUGMSG( g_DebugTitle,g_DebugMessage ) ;
    key = open_key(root, subkey);
	//PHKEY phkResult;

	// THK 27/2/98 - For Part Adviser 2 (intel/dec alpha) need to remove the registry keys setup by Part Adviser
	// when it is run
	if (product == MF_PTA2)
	{
		//MessageBox ( NULL, "PTA2", "Here1", MB_OK ) ;
		RegDeleteKeyNT ( HKEY_CURRENT_USER, "Software\\Moldflow\\Part Adviser 2.00" ) ;
	}
	else if (product == MF_PTA2_PROE)
	{
		RegDeleteKeyNT ( HKEY_CURRENT_USER, "Software\\Parametric Technology Corporation\\Pro/PLASTIC Advisor 2.01" ) ;
	}

	else if (product == MF_PTA2_1)
	{
		RegDeleteKeyNT ( HKEY_CURRENT_USER, "Software\\Moldflow\\Part Adviser 2.10" ) ;
	}

    if(!key) 
	{
        do_error(error, "Couldn't open Environment key");
        return;
    }
    
	DWORD lType = REG_EXPAND_SZ ;
    error = RegQueryValueEx(key, "PATH", NULL, & lType,(unsigned char *)oldpath, &size);

	sprintf ( g_DebugTitle,   "Path Details" ) ;
	sprintf ( g_DebugMessage, "Path is %s",oldpath);
	DEBUGMSG( g_DebugTitle,g_DebugMessage ) ;
	 
    if(error != ERROR_SUCCESS) 
	{
        do_error(error, "Couldn't get value of PATH environment variable");
        return;
    }
     
    newpath[0] = '\0';
    ii = strtok(oldpath, ";");
	char nonquotedir [ MAX_PATH ] ;
	if ( strstr ( dir, "\"" ) )
	{
		strcpy ( nonquotedir, & dir[1] ) ;
		nonquotedir [ strlen ( nonquotedir ) - 1 ] = '\0' ;
	} ;
	sprintf ( g_DebugTitle,   "Quoted dir" ) ;
	sprintf ( g_DebugMessage, "Quoted %s",dir);
	DEBUGMSG( g_DebugTitle,g_DebugMessage ) ;

	sprintf ( g_DebugTitle,   "Non-Quoted dir" ) ;
	sprintf ( g_DebugMessage, "Non-Quoted %s",nonquotedir);
	DEBUGMSG( g_DebugTitle,g_DebugMessage ) ;

    while(ii) 
	{
        if (strcmpi(ii,dir)==0)
		{
        }  
		else if (strcmpi(ii,nonquotedir)==0)
		{
		}
        else
            sprintf(newpath + strlen(newpath), "%s;", ii);
        ii = strtok(NULL, ";");
     }
        jj = strlen(newpath);
        if(jj>0)
            newpath[jj-1] = '\0'; // stomp on final semi-colon
    
		sprintf ( g_DebugTitle,   "New Path Details" ) ;
		sprintf ( g_DebugMessage, "New Path is %s",newpath);
		DEBUGMSG( g_DebugTitle,g_DebugMessage ) ;

		sprintf ( g_DebugTitle,   "Uninstall Dir" ) ;
		sprintf ( g_DebugMessage, "Uninstall Dir is %s",dir);
		DEBUGMSG( g_DebugTitle,g_DebugMessage ) ;
		
		/* REG_EXPAND_SZ */
		if ( newpath[0]==';' )
			error = RegSetValueEx(key, "PATH", 0, REG_EXPAND_SZ, (const unsigned char *)newpath[1], jj);
		else
			error = RegSetValueEx(key, "PATH", 0, REG_EXPAND_SZ, (const unsigned char *)newpath, jj);

}


int get_path()
{
    HKEY        key;
    DWORD       size = BUFFER_SIZE;
    char        *ii;
	//char l_Caption2[82];
	//char l_Message2[82];
    
	
	switch(product)
	{
	case(MF_PTA2):
		//Part Adviser 2
		key = open_key(HKEY_LOCAL_MACHINE,"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\UnInstall\\Part Adviser 2");
		break;
	case(MF_PTA2_PROE):
		//PROE PLASTIC Advisor 2
		key = open_key(HKEY_LOCAL_MACHINE,"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\UnInstall\\Pro PLASTIC Advisor 2.01");
		break;
	case(MF_PTA2_1):
		//Part Adviser 2.1
		key = open_key(HKEY_LOCAL_MACHINE,"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\UnInstall\\Part Adviser 2.10");
		break;
	
    }
    
    
    if(!key) 
	{
        do_error(error, "Couldn't open uninstall key");
        return 1;
    }

	DWORD lType = REG_EXPAND_SZ ;
    error = RegQueryValueEx(key, "UninstallString", NULL, & lType,(unsigned char*) dir, &size);
    
    if(error != ERROR_SUCCESS) 
	{
        do_error(error, "Couldn't get value of Uninstall key");
        return -1;
    } 
	else 
	{
      // Find path, dir = "... -c\"TARGETDIR\\uninst.dll\""
      ii = strstr(dir, "-c");

	  // Move TARGETDIR up to start of "dir"array, move past "-c" 
      if(ii)
          strcpy(dir, ii+2);
	  
	  // Set pointer ii to last "\\" in dir
      ii = strrchr(dir, '\\');
	
	  // If ii is not null, points to last "\\" in dir, end string after "\\"
      if(ii)
          *ii = '\0'; 
	  
	  // If dir[0] contains \", then dir is a quoted path, append new \"
	  if( dir [ 0 ] = '\"' )
		  strcat ( dir, "\"" ) ;

    }

	//MessageBox (NULL, l_Message2, l_Caption2, MB_OK) ;
    return 0;
}

int munge_autoexecfile()
{
    FILE  *orig, *temp;
    char  path[1024], autoexec[1024], tempfile[1024], bakfile[1024];
	char l_Caption2[82];
	char l_Message2[82];
    DWORD size = sizeof(autoexec);

	sprintf ( l_Caption2, "Value of dir" ) ;
	sprintf ( l_Message2, "dir is %s",dir);
	

    // make "path" a quoted lowercase "dir"
    sprintf(path, "\"%s\"", dir);

	sprintf ( l_Caption2, "Value of path" ) ;
	sprintf ( l_Message2, "path is %s",path);
	//MessageBox (NULL, l_Message2, l_Caption2, MB_OK) ;

	strcpy(autoexec, "C:\\autoexec.bat" ) ;
    strcpy(bakfile, autoexec);
    bakfile[strlen(bakfile)-1] = 'k'; // autoexec.bak

    sprintf(tempfile, "%s.temp", autoexec); // autoexec.bat.temp

    orig = fopen(autoexec, "r");
    if(!orig)
	{
        do_error(GetLastError(), "Can't open Autoexec.bat file");
        return -1;
    }

    temp = fopen(tempfile, "w");
    if(!temp)
	{
        do_error(GetLastError(), "Can't open temp file");
        return -1;
    }

	while ( ! feof ( orig ) ) 
	{
		char line [ 2048 ] = "" ;

		fgets ( line, sizeof ( line ), orig ) ;
		// Part Adviser uses MPAPATH
		if (product==MF_PTA2 || product==MF_PTA2_PROE)
		{
			if ( ( strstr ( line, "PATH"    ) != NULL || strstr ( line, "path"    ) != NULL ) &&
				( strstr ( line, "MPAPATH" ) == NULL && strstr ( line, "mpapath" ) == NULL ) && !strstr(line,"MFPATH" )&& !strstr(line,"mfpath")) 
			{
				char newpath [ 2048 ] = "" ;
				char oldpath [ 2048 ] = "" ;

				// Strip path
				if ( strstr ( line, "PATH" ) ) 
				{
					strcpy ( oldpath, strstr ( line, "PATH" ) + 4 ) ;
				} 
				if ( strstr ( line, "path" ) ) 
				{
					strcpy ( oldpath, strstr ( line, "path" ) + 4 ) ;
				} 
			
				// Clean up path for 'CRLF' or ';'
				if ( strstr ( path, "\n" ) )
				{
					strstr ( path, "\n" ) [ 0 ] = '\0' ;
				}
				if ( strstr ( path, ";"  ) )
				{
					strstr ( path, ";" ) [ 0 ] = '\0' ;
				}

				char * ii = strtok(oldpath, "=;\n");
				while(ii) 
				{
					char tmp [ 1024 ] ;
					sprintf ( tmp, "\"%s\"", ii ) ;
					// Quoted or un-quoted
					if ( strcmpi(ii,path)==0 || strcmpi(tmp,path)==0 )
					{
						
						// Do Nothing
					}
					else
					{
						sprintf ( newpath + strlen ( newpath ), "%s;", ii);
						//MessageBox(NULL, newpath,"New path > ",MB_OK | MB_ICONINFORMATION);
					}
					ii = strtok(NULL, "=;\n");
				}
				int jj = strlen(newpath);
				if(jj>0)
				{
					newpath[jj-1] = '\0'; // stomp on final semi-colon
					//MessageBox(NULL, newpath,"Modified path is",MB_OK | MB_ICONINFORMATION);
					//fprintf ( temp, "PATH=%s\n", newpath ) ;
					if (strstr(newpath, "\n"))
					{
						fprintf ( temp, "PATH=%s", newpath ) ;
					}
					else
					{
						fprintf ( temp, "PATH=%s\n", newpath ) ;
					}
				} 
			}
			else
			{
				//fprintf ( temp, "%s\n", line ) ;
				if (strstr(line, "\n"))
				{
					fprintf ( temp, "%s", line ) ;
				}
				else
				{
					if (strlen(line) > 2)
					{
						fprintf ( temp, "%s\n", line ) ;
					}
				}
				
			} 
		} 

}
		fclose(orig);
		fclose(temp);
        
		// mv autoexec.bat autoexec.bak
		unlink( bakfile); rename(autoexec,  bakfile);

		// mv autoexec.bat.temp autoexec.bat
		unlink(autoexec); rename(tempfile, autoexec);
    
		return 0;
}


extern "C"
{


int UninstInitialize(HWND hwndDlg, HWND hInstance, long int lReserved)
{
    OSVERSIONINFO os;
	char		  name [ 1024 ] = "" ;
	HWND		  child ;
	

    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

    parent = hwndDlg;

	child = GetWindow ( parent, GW_CHILD ) ;

	//MessageBox(NULL,"HELLO","PATHFIX.DLL called", MB_OK);

	while ( child != NULL )
	{
		GetWindowText ( child, name, sizeof ( name ) ) ;
		//MessageBox ( NULL, name, "Child Window", MB_OK ) ;
		if ( strstr ( name, "Part Adviser 2.10"	   )) 
		{
			product = MF_PTA2_1 ;
			//MessageBox (NULL,"Part Adviser 2.1 Uninstall","Product Detected",MB_OK);
		}

		if ( strstr ( name, "Part Adviser 2"	   ) && !(strstr(name, "Part Adviser 2 Dec Alpha")) && !(strstr(name, "Part Adviser 2.10"))) 
		{
			product = MF_PTA2 ;
			//MessageBox (NULL,"Part Adviser 2 Uninstall","Product Detected",MB_OK);
		}
		if ( strstr ( name, "Pro PLASTIC Advisor 2"	   ) ) 
		{
			product = MF_PTA2_PROE ;
			//MessageBox (NULL,"Pro PLASTIC Advisor 2 Uninstall","Product Detected",MB_OK);
		}

		child = GetNextWindow ( child, GW_HWNDNEXT ) ;
	} ;


	//MessageBox ( NULL, "Initialize", "PathFix.DLL", MB_OK ) ;
    // NT or 95?
    if( GetVersionEx(&os) == 0)
        do_error(GetLastError(), "Failed to get platform data");

    // find value of TARGETDIR
    if(get_path() < 0)
        return 0;

    if(os.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
	{

        // -- Windows 95 -- //

		//MessageBox ( NULL, "Win 95 Detected", "PathFix.DLL", MB_OK ) ;
        munge_autoexecfile();

    }
	else if(os.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
                
        // -- Windows NT -- //

		//MessageBox ( NULL, "Win NT Detected", "PathFix.DLL", MB_OK ) ;
        // try global environment
        remove_dir_from_path(HKEY_LOCAL_MACHINE,"SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment", 0);


		// try per-user environment
		remove_dir_from_path(HKEY_CURRENT_USER, "Environment", 1);
		
    
        // Notify all windows of change
        SendMessage(HWND_BROADCAST, WM_SETTINGCHANGE, 0,(LPARAM)"Environment");
    }

    return 0; 
}

int UninstUnInitialize(HWND hwndDlg, HWND hInstance, long int lReserved)
{
    return 0;
}

} ;

void do_error(DWORD error, char *msg)
{
    char *buf, buf2[1024];
    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |FORMAT_MESSAGE_FROM_SYSTEM,parent, error, 0, (LPTSTR) &buf, 0, NULL);
    
    sprintf(buf2, "%s: %s", msg, buf);
    //MessageBox(NULL, buf2, "Message from  uninstaller",MB_OK | MB_ICONINFORMATION);
    LocalFree(buf);
}


DWORD RegDeleteKeyNT(HKEY hStartKey , LPTSTR pKeyName )
// JNH 07/08/1996 MSDN Recursive delete registry key

// A registry key that is opened by an application can be deleted
// without error by another application in both Windows 95 and
// Windows NT. This is by design.
{
   DWORD   dwRtn, dwSubKeyLength;
   LPTSTR  pSubKey = NULL;
   TCHAR   szSubKey[512]; // (512) this should be dynamic.
   HKEY    hKey;
 
   // do not allow NULL or empty key name
   if ( pKeyName &&  lstrlen(pKeyName))
   {
 	  
      if( (dwRtn=RegOpenKeyEx(hStartKey,pKeyName,
         0, KEY_ENUMERATE_SUB_KEYS | DELETE, &hKey )) == ERROR_SUCCESS)
      {
         while (dwRtn == ERROR_SUCCESS )
         {
            dwSubKeyLength = 512;
            dwRtn=RegEnumKeyEx(
								hKey,
								0,       // always index zero
								szSubKey,
								&dwSubKeyLength,
								NULL,
								NULL,
								NULL,
								NULL
								);
 
            if(dwRtn == ERROR_NO_MORE_ITEMS)
            {
 				dwRtn = RegDeleteKey(hStartKey, pKeyName);
				break;
            }
            else if(dwRtn == ERROR_SUCCESS)
			{
				dwRtn=RegDeleteKeyNT(hKey, szSubKey);
			}
         }
         RegCloseKey(hKey);
         // Do not save return code because error
         // has already occurred
      }
   }
   else
   {
      dwRtn = ERROR_BADKEY;
   }
 
   return dwRtn;
}
 
